@@ -18,7 +18,7 @@ module Language.C.Analysis.DeclAnalysis (
   mergeOldStyle,
   -- * Dissecting type specs
   canonicalTypeSpec, NumBaseType(..),SignSpec(..),SizeMod(..),NumTypeSpec(..),TypeSpecAnalysis(..),
-  canonicalStorageSpec, StorageSpec(..),hasThreadLocalSpec, isTypeDef,
+  canonicalStorageSpec, StorageSpec(..), hasThreadLocalSpec, isTypeDef,
   -- * Helpers
   VarDeclInfo(..),
   tAttr,mkVarName,getOnlyDeclr,nameOfDecl,analyseVarDecl,analyseVarDecl'
@@ -30,17 +30,14 @@ import Language.C.Data.Ident
 import Language.C.Pretty
 import Language.C.Syntax
 import {-# SOURCE #-} Language.C.Analysis.AstAnalysis (tExpr, ExprSide(..))
-import Language.C.Analysis.DefTable (TagFwdDecl(..), insertType, lookupType)
-import Language.C.Analysis.Export
+import Language.C.Analysis.DefTable (TagFwdDecl(..), insertType)
 import Language.C.Analysis.SemError
 import Language.C.Analysis.SemRep
 import Language.C.Analysis.TravMonad
 
 import Data.Foldable as F (foldrM)
-import qualified Data.Traversable as T
-import Control.Monad (liftM,when,ap)
-import Data.List (intersperse, mapAccumL)
-import Data.Map (Map)
+import Control.Monad (liftM,when,ap,unless,zipWithM)
+import Data.List (intercalate, mapAccumL)
 import qualified Data.Map as Map
 import Text.PrettyPrint.HughesPJ
 
@@ -50,16 +47,19 @@ import Text.PrettyPrint.HughesPJ
 -- | analyse and translate a parameter declaration
 -- Should be called in either prototype or block scope
 tParamDecl :: (MonadTrav m) => CDecl -> m ParamDecl
+tParamDecl (CStaticAssert _ _ node) =
+  astError node "expected parameter, not static assertion"
 tParamDecl (CDecl declspecs declrs node) =
   do declr <- getParamDeclr
      -- analyse the variable declaration
-     (VarDeclInfo name is_inline storage_spec attrs ty declr_node) <- analyseVarDecl' True declspecs declr [] Nothing
-     when (is_inline) $ throwTravError (badSpecifierError node "parameter declaration with inline specifier")
+     (VarDeclInfo name fun_spec  storage_spec attrs ty declr_node) <- analyseVarDecl' True declspecs declr [] Nothing
+     when (isInline fun_spec || isNoreturn fun_spec) $
+       throwTravError (badSpecifierError node "parameter declaration with function specifier")
      -- compute storage of parameter (NoStorage, but might have a register specifier)
      storage <- throwOnLeft $ computeParamStorage node storage_spec
      let paramDecl = mkParamDecl name storage attrs ty declr_node
      -- XXX: we shouldn't modify the deftable here, just analyse and build representation
-     return $ paramDecl
+     return paramDecl
   where
   getParamDeclr =
       case declrs of
@@ -67,7 +67,7 @@ tParamDecl (CDecl declspecs declrs node) =
           [(Just declr,Nothing,Nothing)] -> return declr
           _ -> astError node "bad parameter declaration: multiple decls / bitfield or initializer present"
   mkParamDecl name storage attrs ty declr_node =
-    let vd = VarDecl name (DeclAttrs False storage attrs) ty in
+    let vd = VarDecl name (DeclAttrs noFunctionAttrs storage attrs) ty in
     case name of
       NoName -> AbstractParamDecl vd declr_node
       _ -> ParamDecl vd declr_node
@@ -81,39 +81,45 @@ computeParamStorage node spec       = Left . badSpecifierError node $ "Bad stora
 -- | analyse and translate a member declaration
 tMemberDecls :: (MonadTrav m) => CDecl -> m [MemberDecl]
 -- Anonymous struct or union members
+-- TODO storage specs, align specs and attributes are ignored
+tMemberDecls (CStaticAssert _ _ node) =
+  astError node "expected struct or union member, found static assertion"
 tMemberDecls (CDecl declspecs [] node) =
-  do let (storage_specs, _attrs, typequals, typespecs, is_inline) =
+  do let (_storage_specs, _attrs, typequals, typespecs, funspecs, _alignspecs) =
            partitionDeclSpecs declspecs
-     when is_inline $ astError node "member declaration with inline specifier"
+     unless (null funspecs) $ astError node "member declaration with function specifier"
      canonTySpecs <- canonicalTypeSpec typespecs
      ty <- tType True node typequals canonTySpecs [] []
      case ty of
        DirectType (TyComp _) _ _ ->
          return $ [MemberDecl
                    -- XXX: are these DeclAttrs correct?
-                   (VarDecl NoName (DeclAttrs False NoStorage []) ty)
+                   (VarDecl NoName (DeclAttrs noFunctionAttrs NoStorage []) ty)
                    Nothing node]
        _ -> astError node "anonymous member has a non-composite type"
 -- Named members
-tMemberDecls (CDecl declspecs declrs node) = mapM (uncurry tMemberDecl) (zip (True:repeat False) declrs)
+tMemberDecls (CDecl declspecs declrs node) = zipWithM tMemberDecl (True:repeat False) declrs
     where
     tMemberDecl handle_sue_def (Just member_declr,Nothing,bit_field_size_opt) =
         -- TODO: use analyseVarDecl here, not analyseVarDecl'
         do var_decl <- analyseVarDecl' handle_sue_def declspecs member_declr [] Nothing
-           let (VarDeclInfo name is_inline storage_spec attrs ty declr_node) = var_decl
+           let (VarDeclInfo name fun_spec storage_spec attrs ty _node_info) = var_decl
            --
-           checkValidMemberSpec is_inline storage_spec
-           return $ MemberDecl (VarDecl name (DeclAttrs False NoStorage attrs) ty) bit_field_size_opt node
+           checkValidMemberSpec fun_spec storage_spec
+           return $ MemberDecl (VarDecl name (DeclAttrs noFunctionAttrs NoStorage attrs) ty)
+                               bit_field_size_opt node
     tMemberDecl handle_sue_def (Nothing,Nothing,Just bit_field_size) =
-        do let (storage_specs, _attrs, typequals, typespecs, is_inline) = partitionDeclSpecs declspecs
-           storage_spec  <- canonicalStorageSpec storage_specs
+        do let (storage_specs, _attrs, typequals, typespecs, _funspecs, _alignspecs) = partitionDeclSpecs declspecs
+           -- TODO: funspecs/alignspecs not yet processed
+           _storage_spec  <- canonicalStorageSpec storage_specs
+           -- TODO: storage_spec not used
            canonTySpecs  <- canonicalTypeSpec typespecs
            typ           <- tType handle_sue_def node typequals canonTySpecs [] []
            --
            return $ AnonBitField typ bit_field_size node
     tMemberDecl _ _ = astError node "Bad member declaration"
-    checkValidMemberSpec is_inline storage_spec =
-        do  when (is_inline)                     $ astError node "member declaration with inline specifier"
+    checkValidMemberSpec fun_spec storage_spec =
+        do  when (fun_spec /= noFunctionAttrs)   $ astError node "member declaration with inline specifier"
             when (storage_spec /= NoStorageSpec) $ astError node "storage specifier for member"
             return ()
 
@@ -126,26 +132,27 @@ hasThreadLocalSpec (StaticSpec b) = b
 hasThreadLocalSpec (ExternSpec b) = b
 hasThreadLocalSpec _  = False
 
-data VarDeclInfo = VarDeclInfo VarName Bool {- is-inline? -} StorageSpec Attributes Type NodeInfo
+data VarDeclInfo = VarDeclInfo VarName FunctionAttrs StorageSpec Attributes Type NodeInfo
 
 analyseVarDecl' :: (MonadTrav m) =>
                   Bool -> [CDeclSpec] ->
-                  CDeclr -> [CDecl] -> (Maybe CInit) -> m VarDeclInfo
+                  CDeclr -> [CDecl] -> Maybe CInit -> m VarDeclInfo
 analyseVarDecl' handle_sue_def declspecs declr oldstyle init_opt =
-  do let (storage_specs, attrs, type_quals, type_specs, inline) =
+  do let (storage_specs, attrs, type_quals, type_specs, funspecs, _alignspecs) =
            partitionDeclSpecs declspecs
      canonTySpecs <- canonicalTypeSpec type_specs
-     analyseVarDecl handle_sue_def storage_specs attrs type_quals canonTySpecs inline
+     -- TODO: alignspecs not yet processed
+     analyseVarDecl handle_sue_def storage_specs attrs type_quals canonTySpecs funspecs
                     declr oldstyle init_opt
 
 -- | analyse declarators
 analyseVarDecl :: (MonadTrav m) =>
                   Bool -> [CStorageSpec] -> [CAttr] -> [CTypeQual] ->
-                  TypeSpecAnalysis -> Bool ->
-                  CDeclr -> [CDecl] -> (Maybe CInit) -> m VarDeclInfo
-analyseVarDecl handle_sue_def storage_specs decl_attrs typequals canonTySpecs inline
+                  TypeSpecAnalysis -> [CFunSpec] ->
+                  CDeclr -> [CDecl] -> Maybe CInit -> m VarDeclInfo
+analyseVarDecl handle_sue_def storage_specs decl_attrs typequals canonTySpecs fun_specs
                (CDeclr name_opt derived_declrs asmname_opt declr_attrs node)
-               oldstyle_params init_opt
+               oldstyle_params _init_opt
     = do -- analyse the storage specifiers
          storage_spec  <- canonicalStorageSpec storage_specs
          -- translate the type into semantic representation
@@ -154,11 +161,11 @@ analyseVarDecl handle_sue_def storage_specs decl_attrs typequals canonTySpecs in
          attrs'       <- mapM tAttr (decl_attrs ++ declr_attrs)
          -- make name
          name         <- mkVarName node name_opt asmname_opt
-         return $ VarDeclInfo name inline storage_spec attrs' typ node
+         return $ VarDeclInfo name function_spec storage_spec attrs' typ node
     where
-        isInlineSpec (CInlineQual _) = True
-        isInlineSpec _ = False
-
+        updateFunSpec (CInlineQual _) f = f { isInline = True }
+        updateFunSpec (CNoreturnQual _) f = f { isNoreturn = True }
+        function_spec = foldr updateFunSpec noFunctionAttrs fun_specs
 
 -- return @True@ if the declarations is a type def
 isTypeDef :: [CDeclSpec] -> Bool
@@ -180,13 +187,16 @@ isTypeDef declspecs = not $ null [ n | (CStorageSpec (CTypedef n)) <- declspecs
 --
 -- We move attributes to the type, as they have no meaning for the abstract declarator
 analyseTypeDecl :: (MonadTrav m) => CDecl -> m Type
+analyseTypeDecl (CStaticAssert _ _ node) =
+  astError node "Expected type declaration, found static assert"
 analyseTypeDecl (CDecl declspecs declrs node)
     | [] <- declrs = analyseTyDeclr (emptyDeclr node)
     | [(Just declr,Nothing,Nothing)] <- declrs = analyseTyDeclr declr
     | otherwise = astError node "Bad declarator for type declaration"
     where
     analyseTyDeclr (CDeclr Nothing derived_declrs Nothing attrs _declrnode)
-        | (not (null storagespec) || inline) = astError node "storage specifier for type declaration"
+        | (not (null storagespec) || not (null funspecs) || not (null alignspecs)) =
+            astError node "storage, function or alignment specifier for type declaration"
         | otherwise                          =
           do canonTySpecs <- canonicalTypeSpec typespecs
              t <- tType True node (map CAttrQual (attrs++attrs_decl) ++ typequals)
@@ -195,7 +205,7 @@ analyseTypeDecl (CDecl declspecs declrs node)
                Just n -> withDefTable (\dt -> (t, insertType dt n t))
                Nothing -> return t
         where
-        (storagespec, attrs_decl, typequals, typespecs, inline) = partitionDeclSpecs declspecs
+        (storagespec, attrs_decl, typequals, typespecs, funspecs, alignspecs) = partitionDeclSpecs declspecs
     analyseTyDeclr _ = astError node "Non-abstract declarator in type declaration"
 
 
@@ -253,10 +263,10 @@ tDirectType handle_sue_def node ty_quals canonTySpec = do
                                                | otherwise -> TyFloating floatType
                     Right intType  -> TyIntegral intType
         TSTypeDef tdr -> return$ TypeDefType tdr quals attrs
-        TSNonBasic (CSUType su _tnode)      -> liftM (baseType . TyComp) $ tCompTypeDecl handle_sue_def su
+        TSNonBasic (CSUType su _tnode)       -> liftM (baseType . TyComp) $ tCompTypeDecl handle_sue_def su
         TSNonBasic (CEnumType enum _tnode)   -> liftM (baseType . TyEnum) $ tEnumTypeDecl handle_sue_def enum
         TSType t                             ->  mergeTypeAttributes node quals attrs t
-        TSNonBasic _ -> astError node "Unexpected typespec"
+        TSNonBasic t -> astError node ("Unexpected typespec: " ++ show t)
 
 -- | Merge type attributes
 --
@@ -267,22 +277,19 @@ tDirectType handle_sue_def node ty_quals canonTySpec = do
 mergeTypeAttributes :: (MonadCError m) => NodeInfo -> TypeQuals -> [Attr] -> Type -> m Type
 mergeTypeAttributes node_info quals attrs typ =
     case typ of
-        DirectType ty_name quals' attrs' -> merge quals' attrs' $ mkDirect ty_name
+        DirectType ty_name quals' attrs' -> merge quals' attrs' $ DirectType ty_name
         PtrType ty quals' attrs'  -> merge quals' attrs' $ PtrType ty
         ArrayType ty array_sz quals' attrs' -> merge quals' attrs' $ ArrayType ty array_sz
-        FunctionType (FunType return_ty params inline) attrs'
--- /FIXME/: This needs review, but checking (null attrs) seems strange here
---            | not (null attrs) -> astError node_info "type qualifiers for function type"
---           | otherwise        -> return$ FunctionType (FunType return_ty params inline (attrs' ++ attrs))
-            -> return$ FunctionType (FunType return_ty params inline)  (attrs' ++ attrs)
+        FunctionType fty attrs'
+             | quals /= noTypeQuals -> astError node_info "type qualifiers for function type"
+             | otherwise            -> return$ FunctionType fty (attrs' ++ attrs)
         TypeDefType tdr quals' attrs'
             -> merge quals' attrs' $ TypeDefType tdr
     where
-    mkDirect ty_name quals' attrs' = DirectType ty_name quals' attrs'
     merge quals' attrs' tyf = return $ tyf (mergeTypeQuals quals quals') (attrs' ++ attrs)
 
 typeDefRef :: (MonadCError m, MonadSymtab m) => NodeInfo -> Ident -> m TypeDefRef
-typeDefRef t_node name = lookupTypeDef name >>= \ty -> return (TypeDefRef name (Just ty) t_node)
+typeDefRef t_node name = lookupTypeDef name >>= \ty -> return (TypeDefRef name ty t_node)
 
 -- extract a struct\/union
 -- we emit @declStructUnion@ and @defStructUnion@ actions
@@ -298,9 +305,9 @@ tCompTypeDecl handle_def (CStruct tag ident_opt member_decls_opt attrs node_info
     let decl = CompTypeRef sue_ref tag' node_info
     handleTagDecl (CompDecl decl)
     -- when handle_def is true, enter the definition
-    when (handle_def) $ do
+    when handle_def $
         maybeM member_decls_opt $ \decls ->
-                tCompType sue_ref tag' decls (attrs') node_info
+                tCompType sue_ref tag' decls attrs' node_info
             >>= (handleTagDef.CompDef)
     return decl
 
@@ -328,7 +335,7 @@ tEnumTypeDecl handle_def (CEnum ident_opt enumerators_opt attrs node_info)
         = do sue_ref <- createSUERef node_info ident_opt
              attrs' <- mapM tAttr attrs
              let decl = EnumTypeRef sue_ref node_info
-             when handle_def $ do
+             when handle_def $
                  maybeM enumerators_opt $ \enumerators ->
                          tEnumType sue_ref enumerators attrs' node_info
                     >>=  (handleTagDef . EnumDef)
@@ -346,7 +353,7 @@ tEnumType sue_ref enumerators attrs node = do
     nextEnumerator memo (ident,e) =
       let (memo',expr) = nextEnrExpr memo e in
       (memo', Enumerator ident expr ty (nodeInfo ident))
-    nextEnrExpr :: (Either Integer (Expr,Integer)) -> Maybe CExpr -> (Either Integer (Expr,Integer), CExpr)
+    nextEnrExpr :: Either Integer (Expr,Integer) -> Maybe CExpr -> (Either Integer (Expr,Integer), CExpr)
     nextEnrExpr (Left i) Nothing = (Left (succ i), intExpr i)
     nextEnrExpr (Right (e,offs)) Nothing = (Right (e, succ offs), offsExpr e offs)
     nextEnrExpr _ (Just e) = (Right (e,1), e)
@@ -399,12 +406,13 @@ tArraySize (CArrSize static szexpr) = liftM (ArraySize static) (return szexpr)
 
 tTypeQuals :: (MonadTrav m) => [CTypeQual] -> m (TypeQuals,Attributes)
 tTypeQuals = foldrM go (noTypeQuals,[]) where
-    go (CConstQual _) (tq,attrs) = return$ (tq { constant = True },attrs)
-    go (CVolatQual _) (tq,attrs) = return$ (tq { volatile = True },attrs)
-    go (CRestrQual _) (tq,attrs) = return$ (tq { restrict = True },attrs)
+    go (CConstQual _) (tq,attrs) = return (tq { constant = True },attrs)
+    go (CVolatQual _) (tq,attrs) = return (tq { volatile = True },attrs)
+    go (CRestrQual _) (tq,attrs) = return (tq { restrict = True },attrs)
+    go (CAtomicQual _) (tq,attrs) = return (tq { atomic = True },attrs)
     go (CAttrQual attr) (tq,attrs) = liftM (\attr' -> (tq,attr':attrs)) (tAttr attr)
-    go (CInlineQual node) (_tq,_attrs) = astError node "unexpected inline qualifier"
-
+    go (CNullableQual _) (tq,attrs) = return (tq { nullable = True }, attrs)
+    go (CNonnullQual _) (tq,attrs) = return (tq { nonnull = True }, attrs)
 
 -- * analysis
 
@@ -431,12 +439,9 @@ canonicalTypeSpec = foldrM go TSNone where
     updLongMod NoSizeMod = Just LongMod
     updLongMod LongMod   = Just LongLongMod
     updLongMod _         = Nothing
-    getTypeSpecs :: MonadTrav m => Type -> m [CTypeSpec]
-    getTypeSpecs         =  return . getTS . partitionDeclSpecs . fst . exportType
-    getTS (_, _, _, ts, _) = ts
     go :: (MonadTrav m) => CTypeSpec -> TypeSpecAnalysis -> m TypeSpecAnalysis
-    go (CVoidType _)    TSNone = return$  TSVoid
-    go (CBoolType _)    TSNone = return$  TSBool
+    go (CVoidType _)    TSNone = return TSVoid
+    go (CBoolType _)    TSNone = return TSBool
     go (CCharType _)    tsa | (Just nts@(NumTypeSpec { base = NoBaseType })) <- getNTS tsa
                             = return$  TSNum$ nts { base = BaseChar }
     go (CIntType _)     tsa | (Just nts@(NumTypeSpec { base = NoBaseType })) <- getNTS tsa
@@ -459,17 +464,19 @@ canonicalTypeSpec = foldrM go TSNone where
     go (CComplexType _) tsa | (Just nts@(NumTypeSpec { isComplex = False })) <- getNTS tsa
                             = return$  TSNum$ nts { isComplex = True }
     go (CTypeDef i ni) TSNone = liftM TSTypeDef $ typeDefRef ni i
-    go (CTypeOfType d ni) TSNone = liftM TSType $ analyseTypeDecl d
+    go (CTypeOfType d _ni) TSNone = liftM TSType $ analyseTypeDecl d
     go (CTypeOfExpr e _) TSNone = liftM TSType $ tExpr [] RValue e
+    -- todo: atomic qualifier discarded
+    go (CAtomicType d _ni) TSNone = liftM TSType $ analyseTypeDecl d
     go otherType  TSNone    = return$  TSNonBasic otherType
     go ty _ts = astError (nodeInfo ty) "Invalid type specifier"
 
 -- compute storage given storage specifiers
 canonicalStorageSpec :: (MonadCError m) =>[CStorageSpec] -> m StorageSpec
 canonicalStorageSpec storagespecs = liftM elideAuto $ foldrM updStorage NoStorageSpec storagespecs where
-        updStorage (CAuto _) NoStorageSpec     = return$ AutoSpec
-        updStorage (CRegister _) NoStorageSpec = return$ RegSpec
-        updStorage (CThread _) NoStorageSpec   = return$ ThreadSpec
+        updStorage (CAuto _) NoStorageSpec     = return AutoSpec
+        updStorage (CRegister _) NoStorageSpec = return RegSpec
+        updStorage (CThread _) NoStorageSpec   = return ThreadSpec
         updStorage (CThread _) (StaticSpec _)  = return$ StaticSpec True
         updStorage (CThread _) (ExternSpec _)  = return$ ExternSpec True
         updStorage (CStatic _) NoStorageSpec   = return$ StaticSpec False
@@ -505,9 +512,9 @@ mergeOldStyle node oldstyle_params (CFunDeclr params attrs fdnode : dds) =
             oldstyle_params' <- liftM concat $ mapM splitCDecl oldstyle_params
             param_map <- liftM Map.fromList $ mapM attachNameOfDecl oldstyle_params'
             (newstyle_params,param_map') <- foldrM insertParamDecl ([],param_map) list
-            when (not $ Map.null param_map') $
-                astError node $ "declarations for parameter(s) "++ showParamMap param_map' ++" but no such parameter"
-            return $ (CFunDeclr (Right (newstyle_params, False)) attrs fdnode : dds)
+            unless (Map.null param_map') $
+              astError node $ "declarations for parameter(s) "++ showParamMap param_map' ++" but no such parameter"
+            return (CFunDeclr (Right (newstyle_params, False)) attrs fdnode : dds)
         Right _newstyle -> astError node "oldstyle parameter list, but newstyle function declaration"
     where
         attachNameOfDecl decl = nameOfDecl decl >>= \n -> return (n,decl)
@@ -516,35 +523,38 @@ mergeOldStyle node oldstyle_params (CFunDeclr params attrs fdnode : dds) =
                 Just p -> return (p:ps, Map.delete param_name param_map)
                 Nothing -> return (implicitIntParam param_name : ps, param_map)
         implicitIntParam param_name =
-            let node = (nodeInfo param_name) in
-            CDecl [CTypeSpec (CIntType node)] [(Just (CDeclr (Just param_name) [] Nothing [] node),Nothing,Nothing)] node
-        showParamMap = concat . intersperse ", " . map identToString . Map.keys
+            let nInfo = nodeInfo param_name in
+            CDecl [CTypeSpec (CIntType nInfo)] [(Just (CDeclr (Just param_name) [] Nothing [] nInfo),Nothing,Nothing)] nInfo
+        showParamMap = intercalate ", " . map identToString . Map.keys
 mergeOldStyle node _ _ = astError node "oldstyle parameter list, but not function type"
 
 -- | split a CDecl into declarators, hereby eliding SUE defs from the second declarator on.
 --
---   There are several reasons why this isn;t the preferred way for handling multiple-declarator declarations,
+--   There are several reasons why this isn't the preferred way for handling multiple-declarator declarations,
 --   but it can be convinient some times.
 --
 -- > splitCDecl [d| struct x { int z; } a,*b; |]
--- > [ [d| struct x { int z; } a, struct x *b |] ]
+-- > [ [d| struct x { int z; } a, struct x *b; |] ]
 --
 -- /TODO/: This could be moved to syntax, as it operates on the AST only
 splitCDecl :: (MonadCError m) => CDecl -> m [CDecl]
+splitCDecl decl@(CStaticAssert _ _ _) = return [decl]
 splitCDecl decl@(CDecl declspecs declrs node) =
     case declrs of
         []      -> internalErr "splitCDecl applied to empty declaration"
-        [declr] -> return [decl]
+        -- single declarator, not need to split
+        [_declr] -> return [decl]
+        -- more than one declarator
         (d1:ds) ->
             let declspecs' = map elideSUEDef declspecs in
             return$ (CDecl declspecs [d1] node) : [ CDecl declspecs' [declr] node | declr <- ds ]
     where
     elideSUEDef declspec@(CTypeSpec tyspec) =
         case tyspec of
-            (CEnumType (CEnum name def attrs enum_node) node) ->
-                CTypeSpec (CEnumType (CEnum name Nothing [] enum_node) node)
-            (CSUType (CStruct tag name def attrs su_node) node) ->
-                CTypeSpec (CSUType (CStruct tag name Nothing [] su_node) node)
+            (CEnumType (CEnum name _def _attrs enum_node) node_info) ->
+                CTypeSpec (CEnumType (CEnum name Nothing [] enum_node) node_info)
+            (CSUType (CStruct tag name _def _attrs su_node) node_info) ->
+                CTypeSpec (CSUType (CStruct tag name Nothing [] su_node) node_info)
             _ -> declspec
     elideSUEDef declspec = declspec
 
@@ -559,17 +569,18 @@ tAttr (CAttr name cexpr node) = return$ Attr name cexpr node
 -- TODO: more or less bogus
 mkVarName :: (MonadCError m, MonadSymtab m) =>
              NodeInfo -> Maybe Ident -> Maybe AsmName -> m VarName
-mkVarName  node Nothing _ = return NoName
-mkVarName  node (Just n) asm = return $ VarName n asm
+mkVarName  _node Nothing _ = return NoName
+mkVarName  _node (Just n) asm = return $ VarName n asm
 
 -- helpers
 nameOfDecl :: (MonadCError m) => CDecl -> m Ident
 nameOfDecl d = getOnlyDeclr d >>= \declr ->
     case declr of
-        (CDeclr (Just name) _ _ _ _) -> return name
-        (CDeclr Nothing _ _ _ node) -> internalErr "nameOfDecl: abstract declarator"
+        (CDeclr (Just name) _ _ _ _node) -> return name
+        (CDeclr Nothing _ _ _ _node)     -> internalErr "nameOfDecl: abstract declarator"
 emptyDeclr :: NodeInfo -> CDeclr
 emptyDeclr node = CDeclr Nothing [] Nothing [] node
 getOnlyDeclr :: (MonadCError m) => CDecl -> m CDeclr
 getOnlyDeclr (CDecl _ [(Just declr,_,_)] _) = return declr
-getOnlyDeclr (CDecl _ _ node) = internalErr "getOnlyDeclr: declaration doesn't have a unique declarator"
+getOnlyDeclr (CDecl _ _ _node) = internalErr "getOnlyDeclr: declaration doesn't have a unique declarator"
+getOnlyDeclr (CStaticAssert _ _ _) = internalErr "getOnlyDeclr: static assertion doesn't have a unique declarator"
