@@ -17,7 +17,8 @@
 module Language.C.Data.Ident (
     Ident(..),
     SUERef(..), isAnonymousRef,
-    mkIdent, builtinIdent, internalIdent, internalIdentAt, isInternalIdent, identToString, dumpIdent)
+    mkIdent, builtinIdent, internalIdent, internalIdentAt, isInternalIdent,
+    identToString, sueRefToString, dumpIdent)
 where
 
 -- TODO (comment from manuel):
@@ -26,7 +27,7 @@ where
 import Data.Char
 import Language.C.Data.Position
 import Language.C.Data.Node
-import Language.C.Data.Name   (Name,nameId)
+import Language.C.Data.Name (Name)
 import Data.Generics
 
 -- | References uniquely determining a struct, union or enum type.
@@ -123,6 +124,11 @@ isInternalIdent (Ident _ _ nodeinfo) = isInternalPos (posOfNode nodeinfo)
 identToString               :: Ident -> String
 identToString (Ident s _ _)  = s
 
+-- | string of a SUE ref (empty if anonymous)
+sueRefToString                 :: SUERef -> String
+sueRefToString (AnonymousRef _) = ""
+sueRefToString (NamedRef ident) = identToString ident
+
 -- | dump the identifier string and its positions for debugging purposes
 dumpIdent     :: Ident -> String
 dumpIdent ide  = identToString ide ++ " at " ++ show (nodeInfo ide)
